@@ -8,5 +8,7 @@ var dir_68267d1309a1af8e8297ef4c3efbcdba =
     [ "linalg_factor.f90", "linalg__factor_8f90_source.html", null ],
     [ "linalg_solve.f90", "linalg__solve_8f90_source.html", null ],
     [ "linalg_sorting.f90", "linalg__sorting_8f90_source.html", null ],
-    [ "qrupdate.f90", "qrupdate_8f90_source.html", null ]
+    [ "linalg_sparse.f90", "linalg__sparse_8f90_source.html", null ],
+    [ "qrupdate.f90", "qrupdate_8f90_source.html", null ],
+    [ "sparskit.f90", "sparskit_8f90_source.html", null ]
 ];
\ No newline at end of file
