@@ -2,5 +2,6 @@ var namespaces_dup =
 [
     [ "blas", "namespaceblas.html", "namespaceblas" ],
     [ "linalg", "namespacelinalg.html", "namespacelinalg" ],
-    [ "qrupdate", "namespaceqrupdate.html", "namespaceqrupdate" ]
+    [ "qrupdate", "namespaceqrupdate.html", "namespaceqrupdate" ],
+    [ "sparskit", "namespacesparskit.html", "namespacesparskit" ]
 ];
\ No newline at end of file
